package org.bitbucket.bradleysmithllc.etlunit.feature.database.db;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.google.gson.stream.JsonWriter;
import org.bitbucket.bradleysmithllc.etlunit.util.StringUtils;

import java.io.IOException;
import java.util.*;

public class SchemaImpl implements Schema
{
	private String name;
	private final Catalog catalog;
	private boolean virtual = false;

	private final List<Table> tableList = new ArrayList<Table>();
	private final Map<String, Table> tableMap = new HashMap<String, Table>();

	public SchemaImpl(Catalog catalog, String name) {
		this.name = name;
		this.catalog = catalog;
	}

	public SchemaImpl(Catalog catalog) {
		this.catalog = catalog;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public void setVirtual(boolean virtual) {
		this.virtual = virtual;
	}

	@Override
	public boolean isVirtual() {
		return virtual;
	}

	@Override
	public List<Table> getTables() {
		return Collections.unmodifiableList(tableList);
	}

	@Override
	public Map<String, Table> getTableMap() {
		return Collections.unmodifiableMap(tableMap);
	}

	@Override
	public Table getTable(String name) {
		return tableMap.get(StringUtils.getSafeNullableUppercase(name));
	}

	public void addTable(Table table)
	{
		tableList.add(table);
		tableMap.put(StringUtils.getSafeNullableUppercase(table.getName()), table);

		touch();
	}

	private void touch() {
		if (catalog != null)
		{
			catalog.touch();
		}
	}

	public String toString()
	{
		return (catalog == null ? "" : (catalog.toString() + ".")) + (name == null ? "[DEFAULT]" : name);
	}

	@Override
	public void toJson(JsonWriter writer) throws IOException
	{
		writer.name("name");

		if (name == null)
		{
			writer.nullValue();
		}
		else
		{
			writer.value(name);
		}

		writer.name("virtual").value(virtual);
		writer.name("tables").beginArray();

		for (Table table : tableList)
		{
			writer.beginObject();
			table.toJson(writer);
			writer.endObject();
		}

		writer.endArray();
	}

	@Override
	public void fromJson(JsonNode node)
	{
		JsonNode jsonNode = node.get("name");

		if (!jsonNode.isNull())
		{
			name = jsonNode.asText();
		}

		virtual = node.get("virtual").asBoolean();
		JsonNode tables = node.get("tables");

		if (tables != null && !tables.isNull())
		{
			ArrayNode nodeList = (ArrayNode) tables;

			Iterator<JsonNode> it = nodeList.iterator();

			while (it.hasNext())
			{
				JsonNode tnode = it.next();

				TableImpl timpl = new TableImpl(catalog, this);

				timpl.fromJson(tnode);

				addTable(timpl);
			}
		}
	}
}